#! /usr/bin/wish
#
# tkmpg - a nifty little GUI front-end for mpg123
#
#  This is licensed under the GPL.
#
#  tkmpg.tcl 1.1  :  19 September 1999
#
# Mark Anacker	anacker@ricochet.net
# minor alterations by Todd Lyons  cannonball@usinter.net
#
# I wrote this little hack because I wanted to be able to launch
# mpg123 from a broswer (Netscape) via the nslisten.pl launcher
# by jeremy@theory.org, but there's a little problem.  mpg123
# (my preferred icecast player) isn't a GUI app, so nslisten.pl
# neatly drops it into the background.  Then when I want to stop
# listening, I have to fire up a terminal window, find the pid of
# mpg123, then kill it.
#
# I'm lazy.
#
# So this wrapper gets called by nslisten.pl (or anything else, for
# that matter), with the URL of the stream as a quoted argument.  It
# then builds a minimal UI, launches mpg123 in the background, and sits
# there on my desktop.  When I'm done listening, I hit the Quit button,
# and the callback function nukes the mpg123 process for me.
#
# Simple.
#
# Here's the line I use in nslisten.pl to launch this script:
#
# my $command = "tkmpg.tcl http://$ip:$port";
#
# You can specify the path, as in /home/todd/bin/tkmpg.tcl.  I simply
# placed it in my path (/home/todd/bin).
#
# One hassle I did have to work around is where I launch mpg123 below.
# Tcl lets you easily open a pipe to a child process and grab it's
# standard output.  Unfortunately, mpg123 writes all the interesting
# stuff to standard ERROR.  Since I can't find a way to easily grab stderr,
# I had to implement the inelegant (or is that unelegant???) kludge with bash.  
# A real Tcl or Tk programmer would undoubtedly find a better solution.
#

# flag some things as globals
global pi fid

# set a window titlebar with our name and the URL that
# we were passed

wm title . "tkmpg - $argv"

# establish a default frame

frame .frame1

# shutdown function - this kills the background process.
# It is called by the UI Quit button

proc closeit {} {
  global pi
  exec kill $pi
  exit
}

# Here we create some UI elements - a listbox to display the
# output from mpg123, a couple of scrollbars to navigate the
# listbox, and a button to quit this script *and* kill off the
# mpg123 process.

listbox .listbox1 \
                -width 60 \
                -height 4 \
                -xscrollcommand ".scrollbarh set" \
                -yscrollcommand ".scrollbarv set"

scrollbar .scrollbarv \
                -command ".listbox1 yview" \
                -orient v

scrollbar .scrollbarh \
                -command ".listbox1 xview" \
                -orient h

button .button1 \
                -borderwidth 1 \
                -command closeit \
                -text Quit

# this is the event handler - it gets called when a line of text
# is available on the fid.  It then reads the line, and puts it
# at the bottom of the listbox.

proc getmsg {fid} {
  if {[gets $fid line] >= 0} {
    # insert the new line at the bottom of the box
    .listbox1 insert end $line
    # now adjust the vertical slider to the bottom
    .listbox1 yview end
  }
}

# Here we launch the actual player.  I'm redirecting through /bin/bash
# simply because I can't for the life of me figure out a way to grab
# stderr from the pipe, and that's where mpg123 writes it's output.
# If it had written to stdout, or if Tcl's open command had an option to
# return stderr instead of stdout, then we wouldn't have to go through
# all this.  If your system doesn't support bash, you will want to change
# the line below.  You can eliminate the shell redirect entirely if you
# want - the only effect will be that the listbox doesn't get populated by
# the messages from mpg123

# Mpg123 should be in the path, but if it's not, uncomment the next 2 lines
# and comment the third line.  The first line is where you specify the path
# to your mpg123.
#
# set mpgdir /usr/bin
# set fid [open "| /bin/bash -c \" exec $mpgdir/mpg123 -k 40 -b 4000 $argv 2>&1 \"" "r"]
set fid [open "| /bin/bash -c \" exec mpg123 -k 40 -b 4000 $argv 2>&1 \"" "r"]

# set the input channel to be line buffered
fconfigure $fid -buffering line

# now install our event handler for the lines coming from mpg123
fileevent $fid readable "getmsg $fid"

# save the PID of the background thread for later use when we exit
set pi [pid $fid]

# pack the UI elements in a neat arrangement
grid .listbox1 -row 0 -column 0 -sticky w
grid .scrollbarv -row 0 -column 1 -sticky ns
grid .scrollbarh -row 1 -column 0 -sticky ew
grid .button1 -row 2 -column 0 -sticky ew


